@@ -1,5 +1,6 @@
 /* GDK - The GIMP Drawing Kit
  * Copyright (C) 1995-1997 Peter Mattis, Spencer Kimball and Josh MacDonald
+ * Copyright (C) 1998-2002 Tor Lillqvist
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -28,16 +29,28 @@
 #include "gdkprivate-win32.h"
 
 HWND              gdk_root_window = NULL;
-gint              gdk_event_func_from_window_proc = FALSE;
 HDC		  gdk_display_hdc;
 HINSTANCE	  gdk_dll_hinstance;
 HINSTANCE	  gdk_app_hmodule;
-UINT		  gdk_selection_notify_msg;
-UINT		  gdk_selection_request_msg;
-UINT		  gdk_selection_clear_msg;
-GdkAtom		  gdk_clipboard_atom;
-GdkAtom		  gdk_win32_dropfiles_atom;
-GdkAtom		  gdk_ole2_dnd_atom;
+
+HKL		  _gdk_input_locale;
+UINT		  _gdk_input_codepage;
+
+WORD  		  cf_rtf;
+WORD		  cf_utf8_string;
+
+GdkAtom           utf8_string;
+GdkAtom		  compound_text;
+GdkAtom		  text_uri_list;
+
+GdkAtom		  local_dnd;
+GdkAtom		  gdk_win32_dropfiles;
+GdkAtom		  gdk_ole2_dnd;
+
 GdkAtom           _gdk_selection_property;
-DWORD		  windows_version = 0;
+
+DWORD		  windows_version;
+
 gint		  gdk_input_ignore_wintab = FALSE;
+gint              gdk_event_func_from_window_proc = FALSE;
+gint		  gdk_max_colors = 0;
