@@ -114,6 +114,7 @@ gtk_check_button_init (GtkCheckButton *check_button)
   GTK_WIDGET_SET_FLAGS (check_button, GTK_NO_WINDOW);
   GTK_WIDGET_UNSET_FLAGS (check_button, GTK_RECEIVES_DEFAULT);
   GTK_TOGGLE_BUTTON (check_button)->draw_indicator = TRUE;
+  GTK_BUTTON (check_button)->depress_on_activate = FALSE;
 }
 
 GtkWidget*
@@ -300,9 +301,12 @@ gtk_check_button_size_allocate (GtkWidget     *widget,
 					allocation->width -
 					((border_width + focus_width + focus_pad) * 2
 					 + indicator_size + indicator_spacing * 3));
+	  child_allocation.width = MAX (child_allocation.width, 1);
 
 	  child_allocation.height = MIN (child_requisition.height,
 					 allocation->height - (border_width + focus_width + focus_pad) * 2);
+	  child_allocation.height = MAX (child_allocation.height, 1);
+	  
 	  child_allocation.x = (border_width + indicator_size + indicator_spacing * 3 +
 				widget->allocation.x + focus_width + focus_pad);
 	  child_allocation.y = widget->allocation.y +
@@ -369,75 +373,75 @@ gtk_real_check_button_draw_indicator (GtkCheckButton *check_button,
 				      GdkRectangle   *area)
 {
   GtkWidget *widget;
+  GtkButton *button;
   GtkToggleButton *toggle_button;
   GtkStateType state_type;
   GtkShadowType shadow_type;
-  GdkRectangle restrict_area;
-  GdkRectangle new_area;
-  gint width, height;
   gint x, y;
   gint indicator_size;
   gint indicator_spacing;
   gint focus_width;
   gint focus_pad;
   gboolean interior_focus;
-  GdkWindow *window;
-  
-  widget = GTK_WIDGET (check_button);
-  toggle_button = GTK_TOGGLE_BUTTON (check_button);
   
   if (GTK_WIDGET_DRAWABLE (check_button))
     {
-      window = widget->window;
-
+      widget = GTK_WIDGET (check_button);
+      button = GTK_BUTTON (check_button);
+      toggle_button = GTK_TOGGLE_BUTTON (check_button);
+  
       gtk_widget_style_get (widget, "interior_focus", &interior_focus,
 			    "focus-line-width", &focus_width, 
 			    "focus-padding", &focus_pad, NULL);
 
       _gtk_check_button_get_props (check_button, &indicator_size, &indicator_spacing);
-						    
-      state_type = GTK_WIDGET_STATE (widget);
-      if (state_type != GTK_STATE_NORMAL &&
-	  state_type != GTK_STATE_PRELIGHT)
-	state_type = GTK_STATE_NORMAL;
-      
-      restrict_area.x = widget->allocation.x + GTK_CONTAINER (widget)->border_width;
-      restrict_area.y = widget->allocation.y + GTK_CONTAINER (widget)->border_width;
-      restrict_area.width = widget->allocation.width - ( 2 * GTK_CONTAINER (widget)->border_width);
-      restrict_area.height = widget->allocation.height - ( 2 * GTK_CONTAINER (widget)->border_width);
-      
-      if (gdk_rectangle_intersect (area, &restrict_area, &new_area))
-	{
-	  if (state_type != GTK_STATE_NORMAL)
-	    gtk_paint_flat_box (widget->style, window, state_type, 
-				GTK_SHADOW_ETCHED_OUT, 
-				area, widget, "checkbutton",
-				new_area.x, new_area.y,
-				new_area.width, new_area.height);
-	}
-      
+
       x = widget->allocation.x + indicator_spacing + GTK_CONTAINER (widget)->border_width;
       y = widget->allocation.y + (widget->allocation.height - indicator_size) / 2;
-      width = indicator_size;
-      height = indicator_size;
 
       if (!interior_focus)
 	x += focus_width + focus_pad;      
 
-      state_type = GTK_WIDGET_STATE (widget) == GTK_STATE_ACTIVE ? GTK_STATE_NORMAL : GTK_WIDGET_STATE (widget);
-      if (GTK_TOGGLE_BUTTON (widget)->inconsistent)
+      if (toggle_button->inconsistent)
 	shadow_type = GTK_SHADOW_ETCHED_IN;
-      else if (GTK_TOGGLE_BUTTON (widget)->active)
+      else if (toggle_button->active)
 	shadow_type = GTK_SHADOW_IN;
       else
 	shadow_type = GTK_SHADOW_OUT;
 
+      if (button->activate_timeout || (button->button_down && button->in_button))
+	state_type = GTK_STATE_ACTIVE;
+      else if (button->in_button)
+	state_type = GTK_STATE_PRELIGHT;
+      else
+	state_type = GTK_STATE_NORMAL;
+      
       if (gtk_widget_get_direction (widget) == GTK_TEXT_DIR_RTL)
-	x = widget->allocation.x + widget->allocation.width - (width + x - widget->allocation.x);
+	x = widget->allocation.x + widget->allocation.width - (indicator_size + x - widget->allocation.x);
 
-      gtk_paint_check (widget->style, window,
+      if (GTK_WIDGET_STATE (toggle_button) == GTK_STATE_PRELIGHT)
+	{
+	  GdkRectangle restrict_area;
+	  GdkRectangle new_area;
+	      
+	  restrict_area.x = widget->allocation.x + GTK_CONTAINER (widget)->border_width;
+	  restrict_area.y = widget->allocation.y + GTK_CONTAINER (widget)->border_width;
+	  restrict_area.width = widget->allocation.width - (2 * GTK_CONTAINER (widget)->border_width);
+	  restrict_area.height = widget->allocation.height - (2 * GTK_CONTAINER (widget)->border_width);
+	  
+	  if (gdk_rectangle_intersect (area, &restrict_area, &new_area))
+	    {
+	      gtk_paint_flat_box (widget->style, widget->window, GTK_STATE_PRELIGHT,
+				  GTK_SHADOW_ETCHED_OUT, 
+				  area, widget, "checkbutton",
+				  new_area.x, new_area.y,
+				  new_area.width, new_area.height);
+	    }
+	}
+      
+      gtk_paint_check (widget->style, widget->window,
 		       state_type, shadow_type,
 		       area, widget, "checkbutton",
-		       x, y, width, height);
+		       x, y, indicator_size, indicator_size);
     }
 }
