@@ -80,6 +80,12 @@ struct _GtkItemFactoryClass
   GtkObjectClass	 object_class;
 
   GHashTable		*item_ht;
+
+  /* Padding for future expansion */
+  void (*_gtk_reserved1) (void);
+  void (*_gtk_reserved2) (void);
+  void (*_gtk_reserved3) (void);
+  void (*_gtk_reserved4) (void);
 };
 
 struct _GtkItemFactoryEntry
