@@ -52,6 +52,7 @@
 #include "gtkthemes.h"
 #include "gtkintl.h"
 #include "gtkiconfactory.h"
+#include "gtkmain.h"
 #include "gtkprivate.h"
 #include "gtksettings.h"
 #include "gtkwindow.h"
@@ -95,10 +96,12 @@ struct _GtkRcContext
 
   gchar *theme_name;
   gchar *key_theme_name;
+  gchar *font_name;
   
   gchar *pixmap_path[GTK_RC_MAX_PIXMAP_PATHS];
 
   gint default_priority;
+  GtkStyle *default_style;
 };
 
 static GtkRcContext *gtk_rc_context_get              (GtkSettings     *settings);
@@ -166,8 +169,6 @@ static void        gtk_rc_parse_pixmap_path_string   (GtkRcContext    *context,
 						      GScanner        *scanner,
 						      const gchar     *pix_path);
 static guint       gtk_rc_parse_module_path          (GScanner        *scanner);
-static void        gtk_rc_parse_module_path_string   (const gchar     *mod_path);
-static guint       gtk_rc_parse_im_module_path       (GScanner        *scanner);
 static guint       gtk_rc_parse_im_module_file       (GScanner        *scanner);
 static guint       gtk_rc_parse_path_pattern         (GtkRcContext    *context,
 						      GScanner        *scanner);
@@ -179,7 +180,6 @@ static void        gtk_rc_clear_hash_node            (gpointer         key,
                                                       gpointer         data,
                                                       gpointer         user_data);
 static void        gtk_rc_clear_styles               (GtkRcContext    *context);
-static void        gtk_rc_append_default_module_path (void);
 static void        gtk_rc_add_initial_default_files  (void);
 
 static void        gtk_rc_style_init                 (GtkRcStyle      *style);
@@ -272,7 +272,6 @@ static const struct
   { "engine", GTK_RC_TOKEN_ENGINE },
   { "module_path", GTK_RC_TOKEN_MODULE_PATH },
   { "stock", GTK_RC_TOKEN_STOCK },
-  { "im_module_path", GTK_RC_TOKEN_IM_MODULE_PATH },
   { "im_module_file", GTK_RC_TOKEN_IM_MODULE_FILE },
   { "LTR", GTK_RC_TOKEN_LTR },
   { "RTL", GTK_RC_TOKEN_RTL }
@@ -280,15 +279,11 @@ static const struct
 
 static GHashTable *realized_style_ht = NULL;
 
-static gchar *im_module_path = NULL;
 static gchar *im_module_file = NULL;
 
 #define GTK_RC_MAX_DEFAULT_FILES 128
 static gchar *gtk_rc_default_files[GTK_RC_MAX_DEFAULT_FILES];
 
-#define GTK_RC_MAX_MODULE_PATHS 128
-static gchar *module_path[GTK_RC_MAX_MODULE_PATHS];
-
 /* A stack of directories for RC files we are parsing currently.
  * these are implicitely added to the end of PIXMAP_PATHS
  */
@@ -304,9 +299,9 @@ gtk_rc_make_default_dir (const gchar *type)
 
   var = g_getenv ("GTK_EXE_PREFIX");
   if (var)
-    path = g_build_filename (var, "lib", "gtk-2.0", type, GTK_BINARY_VERSION, NULL);
+    path = g_build_filename (var, "lib", "gtk-2.0", GTK_BINARY_VERSION, type, NULL);
   else
-    path = g_build_filename (GTK_LIBDIR, "gtk-2.0", type, GTK_BINARY_VERSION, NULL);
+    path = g_build_filename (GTK_LIBDIR, "gtk-2.0", GTK_BINARY_VERSION, type, NULL);
 
   return path;
 }
@@ -317,29 +312,25 @@ gtk_rc_make_default_dir (const gchar *type)
  *    look for IM modules.
  *
  * Obtains the path in which to look for IM modules. See the documentation
- * of the <link linkend="im-module-path"><envar>GTK_IM_MODULE_PATH</envar></link>
- * environment variable for more details.
+ * of the <link linkend="im-module-path"><envar>GTK_PATH</envar></link>
+ * environment variable for more details about looking up modules. This
+ * function is useful solely for utilities supplied with GTK+ and should
+ * not be used by applications under normal circumstances.
  */
 gchar *
 gtk_rc_get_im_module_path (void)
 {
-  const gchar *result = g_getenv ("GTK_IM_MODULE_PATH");
+  gchar **paths = _gtk_get_module_path ("immodules");
+  gchar *result = g_strjoinv (G_SEARCHPATH_SEPARATOR_S, paths);
+  g_strfreev (paths);
 
-  if (!result)
-    {
-      if (im_module_path)
-	result = im_module_path;
-      else
-	return gtk_rc_make_default_dir ("immodules");
-    }
-
-  return g_strdup (result);
+  return result;
 }
 
 /**
  * gtk_rc_get_im_module_file:
  * @returns: a newly-allocated string containing the name of the file
- * listing the IM modules to load
+ * listing the IM modules available for loading
  *
  * Obtains the path to the IM modules file. See the documentation
  * of the <link linkend="im-module-file"><envar>GTK_IM_MODULE_FILE</envar></link>
@@ -376,39 +367,22 @@ gtk_rc_get_theme_dir (void)
   return path;
 }
 
+/**
+ * gtk_rc_get_module_dir:
+ * 
+ * Returns a directory in which GTK+ looks for theme engines.
+ * For full information about the search for theme engines,
+ * see the docs for <envar>GTK_PATH</envar> in
+ * <xref linkend="gtk-running">.
+ * 
+ * return value: the directory. (Must be freed with g_free())
+ **/
 gchar *
 gtk_rc_get_module_dir (void)
 {
   return gtk_rc_make_default_dir ("engines");
 }
 
-static void
-gtk_rc_append_default_module_path (void)
-{
-  const gchar *var;
-  gchar *path;
-  gint n;
-
-  for (n = 0; module_path[n]; n++) ;
-  if (n >= GTK_RC_MAX_MODULE_PATHS - 1)
-    return;
-  
-  var = g_getenv ("GTK_EXE_PREFIX");
-  if (var)
-    path = g_build_filename (var, "lib", "gtk-2.0", GTK_VERSION, "engines", NULL);
-  else
-    path = g_build_filename (GTK_LIBDIR, "gtk-2.0", GTK_VERSION, "engines", NULL);
-  module_path[n++] = path;
-
-  var = g_get_home_dir ();
-  if (var)
-    {
-      path = g_build_filename (var, ".gtk-2.0", GTK_VERSION, "engines", NULL);
-      module_path[n++] = path;
-    }
-  module_path[n] = NULL;
-}
-
 static void
 gtk_rc_add_initial_default_files (void)
 {
@@ -424,7 +398,7 @@ gtk_rc_add_initial_default_files (void)
   gtk_rc_default_files[0] = NULL;
   init = TRUE;
 
-  var = g_getenv ("GTK_RC_FILES");
+  var = g_getenv ("GTK2_RC_FILES");
   if (var)
     {
       files = g_strsplit (var, G_SEARCHPATH_SEPARATOR_S, 128);
@@ -550,6 +524,14 @@ gtk_rc_settings_changed (GtkSettings  *settings,
   g_free (new_key_theme_name);
 }
 
+static void
+gtk_rc_font_name_changed (GtkSettings  *settings,
+                          GParamSpec   *pspec,
+                          GtkRcContext *context)
+{
+  _gtk_rc_context_get_default_font_name (settings);
+}
+
 static GtkRcContext *
 gtk_rc_context_get (GtkSettings *settings)
 {
@@ -563,10 +545,12 @@ gtk_rc_context_get (GtkSettings *settings)
       context->rc_sets_widget_class = NULL;
       context->rc_sets_class = NULL;
       context->rc_files = NULL;
+      context->default_style = NULL;
 
       g_object_get (settings,
 		    "gtk-theme-name", &context->theme_name,
 		    "gtk-key-theme-name", &context->key_theme_name,
+		    "gtk-font-name", &context->font_name,
 		    NULL);
 
       g_signal_connect (settings,
@@ -577,6 +561,11 @@ gtk_rc_context_get (GtkSettings *settings)
 			"notify::gtk-key-theme-name",
 			G_CALLBACK (gtk_rc_settings_changed),
 			context);
+      g_signal_connect (settings,
+			"notify::gtk-font-name",
+			G_CALLBACK (gtk_rc_font_name_changed),
+			context);
+
       
       context->pixmap_path[0] = NULL;
 
@@ -655,9 +644,6 @@ _gtk_rc_init (void)
     {
       initialized = TRUE;
       
-      module_path[0] = NULL;
-      gtk_rc_append_default_module_path();
-      
       gtk_rc_add_initial_default_files ();
     }
   
@@ -1275,6 +1261,72 @@ gtk_rc_reset_widgets (GtkRcContext *context)
   g_list_free (toplevels);
 }
 
+static void
+gtk_rc_clear_realized_style (gpointer key,
+			     gpointer value,
+			     gpointer data)
+{
+  GSList *rc_styles = key;
+  GSList *tmp_list = rc_styles;
+
+  while (tmp_list)
+    {
+      GtkRcStyle *rc_style = tmp_list->data;
+      
+      rc_style->rc_style_lists = g_slist_remove_all (rc_style->rc_style_lists,
+						     rc_styles);
+      tmp_list = tmp_list->next;
+    }
+
+  g_slist_free (rc_styles);
+}
+
+const gchar*
+_gtk_rc_context_get_default_font_name (GtkSettings *settings)
+{
+  GtkRcContext *context;
+  gchar *new_font_name;
+  
+  g_return_val_if_fail (GTK_IS_SETTINGS (settings), NULL);
+
+  context = gtk_rc_context_get (settings);
+
+  g_object_get (context->settings,
+                "gtk-font-name", &new_font_name,
+                NULL);
+
+  if (new_font_name != context->font_name && !(new_font_name && strcmp (context->font_name, new_font_name) == 0))
+    {
+      gboolean reset = FALSE;
+      g_free (context->font_name);
+      context->font_name = g_strdup (new_font_name);
+
+      if (context->default_style)
+        {
+	  g_object_unref (G_OBJECT (context->default_style));
+	  context->default_style = NULL;
+	  reset = TRUE;
+        }
+
+      /* Clear out styles that have been looked up already
+       */
+      if (realized_style_ht)
+	{
+	  g_hash_table_foreach (realized_style_ht, gtk_rc_clear_realized_style, NULL);
+	  g_hash_table_destroy (realized_style_ht);
+	  realized_style_ht = NULL;
+	  reset = TRUE;
+	}
+
+      if (reset)
+	gtk_rc_reset_widgets (context);
+    }
+          
+  g_free (new_font_name);
+
+  return context->font_name;
+}
+
 /**
  * gtk_rc_reparse_all_for_settings:
  * @settings: a #GtkSettings
@@ -1362,6 +1414,7 @@ gtk_rc_reparse_all_for_settings (GtkSettings *settings,
 
       g_free (context->theme_name);
       g_free (context->key_theme_name);
+
       g_object_get (context->settings,
 		    "gtk-theme-name", &context->theme_name,
 		    "gtk-key-theme-name", &context->key_theme_name,
@@ -1548,15 +1601,22 @@ gtk_rc_get_style (GtkWidget *widget)
 
   if (rc_styles)
     return gtk_rc_init_style (rc_styles);
+  else
+    {
+      if (!context->default_style)
+        context->default_style = gtk_style_new ();
 
-  return NULL;
+      return context->default_style;
+    }
 }
 
 /**
  * gtk_rc_get_style_by_paths:
  * @settings: a #GtkSettings object
  * @widget_path: the widget path to use when looking up the style, or %NULL
+ *               if no matching against the widget path should be done
  * @class_path: the class path to use when looking up the style, or %NULL
+ *               if no matching against the class path should be done.
  * @type: a type that will be used along with parent types of this type
  *        when matching against class styles, or #G_TYPE_NONE
  * 
@@ -1597,7 +1657,7 @@ gtk_rc_get_style_by_paths (GtkSettings *settings,
 
   context = gtk_rc_context_get (settings);
 
-  if (context->rc_sets_widget)
+  if (widget_path && context->rc_sets_widget)
     {
       gchar *path_reversed;
       guint path_length;
@@ -1610,7 +1670,7 @@ gtk_rc_get_style_by_paths (GtkSettings *settings,
       g_free (path_reversed);
     }
   
-  if (context->rc_sets_widget_class)
+  if (class_path && context->rc_sets_widget_class)
     {
       gchar *path_reversed;
       guint path_length;
@@ -2265,9 +2325,6 @@ gtk_rc_parse_statement (GtkRcContext *context,
     case GTK_RC_TOKEN_MODULE_PATH:
       return gtk_rc_parse_module_path (scanner);
       
-    case GTK_RC_TOKEN_IM_MODULE_PATH:
-      return gtk_rc_parse_im_module_path (scanner);
-      
     case GTK_RC_TOKEN_IM_MODULE_FILE:
       return gtk_rc_parse_im_module_file (scanner);
 
@@ -2873,28 +2930,20 @@ gtk_rc_find_pixmap_in_path (GtkSettings  *settings,
   return NULL;
 }
 
+/**
+ * gtk_rc_find_module_in_path:
+ * @module_file: name of a theme engine
+ * 
+ * Searches for a theme engine in the GTK+ search path. This function
+ * is not useful for applications and should not be used.
+ * 
+ * Return value: The filename, if found (must be freed with g_free()),
+ *   otherwise %NULL.
+ **/
 gchar*
 gtk_rc_find_module_in_path (const gchar *module_file)
 {
-  gint i;
-  gint fd;
-  gchar *buf;
-  
-  for (i = 0; (i < GTK_RC_MAX_MODULE_PATHS) && (module_path[i] != NULL); i++)
-    {
-      buf = g_build_filename (module_path[i], module_file, NULL);
-      
-      fd = open (buf, O_RDONLY);
-      if (fd >= 0)
-	{
-	  close (fd);
-	  return buf;
-	}
-      
-      g_free (buf);
-    }
-    
-  return NULL;
+  return _gtk_find_module (module_file, "engines");
 }
 
 static guint
@@ -2961,7 +3010,11 @@ gtk_rc_parse_font_name (GScanner   *scanner,
   if (token != G_TOKEN_STRING)
     return G_TOKEN_STRING;
 
-  rc_style->font_desc = pango_font_description_from_string (scanner->value.v_string);
+  if (rc_style->font_desc)
+    pango_font_description_free (rc_style->font_desc);
+
+  rc_style->font_desc = 
+    pango_font_description_from_string (scanner->value.v_string);
   
   return G_TOKEN_NONE;
 }
@@ -3328,29 +3381,8 @@ gtk_rc_parse_module_path (GScanner *scanner)
   token = g_scanner_get_next_token (scanner);
   if (token != G_TOKEN_STRING)
     return G_TOKEN_STRING;
-  
-  gtk_rc_parse_module_path_string (scanner->value.v_string);
-  
-  return G_TOKEN_NONE;
-}
 
-static guint
-gtk_rc_parse_im_module_path (GScanner *scanner)
-{
-  guint token;
-  
-  token = g_scanner_get_next_token (scanner);
-  if (token != GTK_RC_TOKEN_IM_MODULE_FILE)
-    return GTK_RC_TOKEN_IM_MODULE_FILE;
-  
-  token = g_scanner_get_next_token (scanner);
-  if (token != G_TOKEN_STRING)
-    return G_TOKEN_STRING;
-
-  if (im_module_path)
-    g_free (im_module_path);
-    
-  im_module_path = g_strdup (scanner->value.v_string);
+  g_warning ("module_path directive is now ignored\n");
 
   return G_TOKEN_NONE;
 }
@@ -3376,39 +3408,6 @@ gtk_rc_parse_im_module_file (GScanner *scanner)
   return G_TOKEN_NONE;
 }
 
-static void
-gtk_rc_parse_module_path_string (const gchar *mod_path)
-{
-  gint end_offset;
-  gint start_offset = 0;
-  gint path_len;
-  gint path_num;
-  
-  /* free the old one, or just add to the old one ? */
-  for (path_num=0; module_path[path_num]; path_num++)
-    {
-      g_free (module_path[path_num]);
-      module_path[path_num] = NULL;
-    }
-  
-  path_num = 0;
-  
-  path_len = strlen (mod_path);
-  
-  for (end_offset = 0; end_offset <= path_len; end_offset++)
-    {
-      if ((mod_path[end_offset] == G_SEARCHPATH_SEPARATOR) ||
-	  (end_offset == path_len))
-	{
-	  module_path[path_num] = g_strndup (mod_path + start_offset, end_offset - start_offset);
-	  path_num++;
-	  module_path[path_num] = NULL;
-	  start_offset = end_offset + 1;
-	}
-    }
-  gtk_rc_append_default_module_path();
-}
-
 static guint
 gtk_rc_parse_path_pattern (GtkRcContext *context,
 			   GScanner     *scanner)
@@ -3769,12 +3768,13 @@ gtk_rc_parse_stock (GtkRcContext   *context,
         }
     }
 
-  if (icon_set && icon_set_valid)
+  if (icon_set)
     {
-      gtk_icon_factory_add (factory,
-                            stock_id,
-                            icon_set);
-      
+      if (icon_set_valid)
+        gtk_icon_factory_add (factory,
+                              stock_id,
+                              icon_set);
+
       gtk_icon_set_unref (icon_set);
     }
   
