@@ -1168,7 +1168,7 @@ gtk_text_tag_set_property (GObject      *object,
     case PROP_BG_FULL_HEIGHT:
       text_tag->bg_full_height_set = TRUE;
       text_tag->values->bg_full_height = g_value_get_boolean (value);
-      g_object_notify (object, "bg_full_height_set");
+      g_object_notify (object, "background_full_height_set");
       break;
 
     case PROP_LANGUAGE:
@@ -1871,7 +1871,7 @@ gtk_text_attributes_get_type (void)
   static GType our_type = 0;
   
   if (our_type == 0)
-    our_type = g_boxed_type_register_static ("GtkTypeTextAttributes",
+    our_type = g_boxed_type_register_static ("GtkTextAttributes",
 					     (GBoxedCopyFunc) gtk_text_attributes_ref,
 					     (GBoxedFreeFunc) gtk_text_attributes_unref);
 
